#!perl -w
use Test::More tests => 6;
use strict;
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::FileIterator;
use DE_EPAGES::Core::API::File qw( DeleteFile WriteFile );
use DE_EPAGES::Core::API::Script qw( RunScript );

RunScript( Message => 'long', Sub => sub {

    my $FileName = 'FileIterator.txt';
    my $Content = "Line 1\nLine 2";
    WriteFile( $FileName, \$Content );

    my $Iterator = DE_EPAGES::Core::API::FileIterator->new('<', $FileName, sub {
        my $self = shift;
        my( $Line ) = @_;
        ok( $self->isa('DE_EPAGES::Core::API::FileIterator'), 'self' );
        return uc($Line);
    });

    is( $Iterator->next, "LINE 1\n", 'line 1' );
    is( <$Iterator>, 'LINE 2', 'line 2' );

    undef $Iterator; # close the file handle

    DeleteFile( $FileName );

    eval {
        DE_EPAGES::Core::API::FileIterator->new('<', 'nofile');
    };
    ok( ExistsError(), 'error if file does not exist' );
    is( GetError()->code, 'FILE_OPEN_FAILED', 'error code FILE_OPEN_FAILED' );
});
