package TestCustomerAddress;
use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::DHLIntraship::API::Attributes::CustomerAddress;
use Test::More;

sub Main {
    testNormalBehavior();
    testWarning();
    testCountryIsUndef();
    done_testing();
}
sub testCountryIsUndef {
    my $SubName = (caller(0))[3];
    my $ShippingLabelWithCountryUndef = StubShippingLabelCountryUndef();
    my $ShopAddress = DE_EPAGES::DHLIntraship::API::Attributes::CustomerAddress->new( $ShippingLabelWithCountryUndef );
    $ShopAddress->{'AttributeNames'} = [];
    $ShopAddress->{'MandatoryAttributeNames'} = [];

    is($ShopAddress->isComplete(),0,"$SubName - is not comlete because of undef country - EXP: 0");
    is(scalar @{$ShopAddress->getStatesAsWarning()},1, "$SubName - the missing country - EXP: Country")
}

sub testWarning {
    my $SubName = (caller(0))[3];
    my $ShippingLabelWithUndef = StubShippingLabelUndef();
    my $ShopAddress = DE_EPAGES::DHLIntraship::API::Attributes::CustomerAddress->new( $ShippingLabelWithUndef );
    $ShopAddress->{'AttributeNames'} = ['Attribute1'];
    $ShopAddress->{'MandatoryAttributeNames'} = ['UndefButMandatory1','UndefButMandatory2'];

    is($ShopAddress->isComplete(),0,"$SubName - is not comlete because of missing Attributes - EXP: 0");
    is($ShopAddress->{'Attribute1'}, undef,"$SubName - explain what is tested - explain what you espected");
    my $ExpectedWarnings = ['UndefButMandatory1','UndefButMandatory2', 'Code2'];
    is_deeply($ShopAddress->{'WarningStates'}, $ExpectedWarnings, "$SubName - find all warnings - warnings as Array");
    is(scalar @{$ShopAddress->getStatesAsWarning()},3, "$SubName - the missing country - EXP: Country")
}

sub testNormalBehavior {
    my $SubName = (caller(0))[3];
    my $ShippingLabel = StubShippingLabel();
    my $ShopAddress = DE_EPAGES::DHLIntraship::API::Attributes::CustomerAddress->new( $ShippingLabel );
    $ShopAddress->{'AttributeNames'} = ['Attribute1','Attribute2'];
    $ShopAddress->{'MandatoryAttributeNames'} = ['MandatoryAttribute1','MandatoryAttribute2'];

    is($ShopAddress->isComplete(),1,"$SubName - all should be fine - EXP: 1");
    is($ShopAddress->{'Attribute1'},'Attribute1',"$SubName - normalAttribute is stored - EXP: Attributename: Attribute1");
    is($ShopAddress->{'Attribute2'},'Attribute2',"$SubName - normalAttribute is stored - EXP: Attributename: Attribute2");
    is($ShopAddress->{'Attribute3'},undef,"$SubName - whats about undef but not mandatory Attributes - EXP: undef ");
    is($ShopAddress->{'MandatoryAttribute1'},'MandatoryAttribute1',"$SubName - mandatory Attribute is stored - EXP: Attributename: MandatoryAttribute1");
    is($ShopAddress->{'MandatoryAttribute2'},'MandatoryAttribute2',"$SubName - mandatory Attribute is stored - EXP: Attributename: MandatoryAttribute2");
    is($ShopAddress->{'Code2'},'Code2',"$SubName - code2 Attribute is stored - EXP: Attributename: Code2");
    is_deeply($ShopAddress->{'WarningStates'}, [], "$SubName - no warnings bacause all went fine- EXP: Empty array ref");
    is(scalar @{$ShopAddress->getStatesAsWarning()},0 , "$SubName - the missing country - EXP: Country")
}

sub StubShippingLabel {
    {
        ## no critic ( ProhibitNestedSubs ProhibitMultiplePackages)
        package ShippingLabel;
        sub get{
            my $self = shift;
            my ($AttributeName) = @_;
            return $self if $AttributeName eq 'Address';
            return {'Code2' => 'Code2'} if $AttributeName eq 'Country';
            return $AttributeName;
        };
        return bless {}, 'ShippingLabel';
        ## use critic ( ProhibitNestedSubs ProhibitMultiplePackages)
    }
}

sub StubShippingLabelUndef {
    {
        ## no critic ( ProhibitNestedSubs ProhibitMultiplePackages)
        package ShippingLabelUndef;
        sub get{
            my $self = shift;
            my ($AttributeName) = @_;
            return $self if $AttributeName eq 'Address';
            return $self if $AttributeName eq 'Country';
            return undef;
        };
        return bless {}, 'ShippingLabelUndef';
        ## use critic ( ProhibitNestedSubs ProhibitMultiplePackages)
    }
}
sub StubShippingLabelCountryUndef {
    {
        ## no critic ( ProhibitNestedSubs ProhibitMultiplePackages)
        package ShippingLabelCountryUndef;
        sub get{
            my $self = shift;
            my ($AttributeName) = @_;
            return $self if $AttributeName eq 'Address';
            return undef if $AttributeName eq 'Country';
            return $AttributeName;
        };
        return bless {}, 'ShippingLabelCountryUndef';
        ## use critic ( ProhibitNestedSubs ProhibitMultiplePackages)
    }
}

RunScript( Sub => \&Main );
