use strict;

use Data::Dumper;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::DHLIntraship::API::ServicePackageOptionConstants::MappingOptionsOptionGroups qw (
    %OPTIONGROUPS
    GetOptionGroups
    GetServicePackageOptions
);
use Test::More;

our $OPTIONBLOCK1 = ['Option11','Option12','Option13'];
our $OPTIONBLOCK2 = ['Option21','Option22','Option23'];
sub Main {
    %OPTIONGROUPS = (
        'OptionGroup1' => $OPTIONBLOCK1,
        'OptionGroup2' => $OPTIONBLOCK2,
    );
    TestGetOptionGroups();
    TestGetServicePackageOptions();
    done_testing();
}

sub TestGetOptionGroups {
    is_deeply(GetOptionGroups(), [ 'OptionGroup1', 'OptionGroup2' ], 'give the Correct Option group');;
}
sub TestGetServicePackageOptions {
    is_deeply(GetServicePackageOptions(), [@$OPTIONBLOCK1, @$OPTIONBLOCK2], 'give the Correct Package Types');
    is_deeply(GetServicePackageOptions('OptionGroup1'), $OPTIONBLOCK1, 'give the Correct Package Types for ServiceType1');
    is_deeply(GetServicePackageOptions('OptionGroup2'), $OPTIONBLOCK2, 'give the Correct Package Types for ServiceType2');
    eval {
        GetServicePackageOptions('NotAOptionsType');
    };
    if (ExistsError()) {
        my $Error = GetError();
        is($Error->code,'NotAValidOptionsType','correct Error handling');
        is($Error->{'Vars'}->{'Parameter'},'NotAOptionsType','correct Error handling Parameter value');
        LogError($Error);
    }
}

RunScript( Sub => \&Main );