use strict;

use Data::Dumper;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::DHLIntraship::API::ServicePackageOptionConstants::MappingPackageTypeOptions qw (
    %MAPPINGPACKAGETYPEOPTIONS
    GetOption
    GetPackageTypes
);
use Test::More;

my $SOMEBIGVALUE = 2000;

sub Main {
    %MAPPINGPACKAGETYPEOPTIONS = (
        'PackageType1' => {
            'Option11' => 1,
            'Option12' => 1000,
        },
        'PackageType2' => {
            'Option21' => 1,
            'Option22' => $SOMEBIGVALUE,
        },
    );
    TestGetPackageTypes();
    TestGetOption();
    done_testing();
}

sub TestGetPackageTypes {
    is_deeply(GetPackageTypes(), ['PackageType1','PackageType2'], 'give the Correct Package Types');;
}

sub TestGetOption {
    { # mocking focus
    ## no critic (ProtectPrivateVars)
    local *DE_EPAGES::DHLIntraship::API::ServicePackageOptionConstants::MappingPackageTypeOptions::_GetServicePackageOptionsWrapper = sub {
                return ['Option11','Option12','Option21','Option22'];
            };
    ## use critic (ProtectPrivateVars)
    # test correct behavior
    is(GetOption('UnkownPackageType', 'Option22'),undef,'get undef for UnkownPackageType');
    is(GetOption('PackageType2', 'UnknownOption'),undef,'get undef for UnknownOption');
    is(GetOption('PackageType2', 'Option22'),$SOMEBIGVALUE,'Get Option correct behavior');
    }
}

sub HelperErrorCallGetOption{
    my ( $PackageType, $OptionName, $ExpectedErrorCode, $ExpectedErrorValue ) = @_;
    eval {
        GetOption($PackageType, $OptionName);
    };
    if (ExistsError()) {
        my $Error = GetError();
        my $PrintPackageType = $PackageType?$PackageType:'undefined';
        my $PrintOptionName = $OptionName?$OptionName:'undefined';
        is($Error->code,$ExpectedErrorCode,"correct Error handling: PackageType:$PrintPackageType, OptionName:$PrintOptionName");
        is($Error->{'Vars'}->{'Parameter'},$ExpectedErrorValue,"correct Error parameter handling: PackageType:$PrintPackageType, OptionName:$PrintOptionName");
    }
}
RunScript( Sub => \&Main );
1;