package TestProducts;

use strict;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::DHLIntraship::API::Attributes::Products;
use Test::More;

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            PositiveCase();
            NegativeCase();
            # tests
            done_testing();
        }
    );
}
sub PositiveCase {
    my $subTestName = (caller(0))[3];
    my $ShippingLabel = StubShippingLabelDefault();
    my $ProductData = DE_EPAGES::DHLIntraship::API::Attributes::Products->new( $ShippingLabel );
    $ProductData->isComplete();

    ## no critic ( ProhibitMagicNumbers )
    is($ProductData->{'TotalWeight'},11 ,"$subTestName - get the TotalWeight - 11");
    is($ProductData->{'ParcelDepth'},2200 ,"$subTestName - get the ParcelDepth in cm- 2.2");
    is($ProductData->{'ParcelLength'},3300 ,"$subTestName - get the ParcelLength in cm- 3.3");
    is($ProductData->{'ParcelWidth'},4400 ,"$subTestName - get the ParcelWidth in cm- 4.4");
    is($ProductData->{'LineItemsSum'},666 ,"$subTestName - get the LineItemsSum in cm- 4.4");
   is(scalar @{$ProductData->getStatesAsWarning()},0 , "$subTestName - empty array of warnings - 0")
    ## use critic ( ProhibitMagicNumbers )
}

sub NegativeCase {
    my $subTestName = (caller(0))[3];
    my $ShippingLabel = StubShippingLabelEmpty();
    my $ProductData = DE_EPAGES::DHLIntraship::API::Attributes::Products->new( $ShippingLabel );
    $ProductData->isComplete();

   is(scalar @{$ProductData->getStatesAsWarning()}, 1 , "$subTestName - get a list of warnings - 1");
}

sub StubShippingLabelDefault {
    {
        ## no critic ( ProhibitNestedSubs ProhibitMultiplePackages ProhibitMagicNumbers )
        package StubShippingLabelDefault;
        sub get{
            my $self = shift;
            my ($AttributeName) = @_;
            return 11 if $AttributeName eq 'TotalWeight';
            return 2 if $AttributeName eq 'TotalAmount';
            return $self if $AttributeName eq 'LineItemContainer';
            return 666 if $AttributeName eq 'LineItemsSum';
            # in mm
            return 22 if $AttributeName eq 'ParcelDepth';
            return 33 if $AttributeName eq 'ParcelLength';
            return 44 if $AttributeName eq 'ParcelWidth';
            # factor for mm
            return 0.001 if $AttributeName eq 'Factor';
            return 'ServiceType' if $AttributeName eq 'ServiceGroup';
            return undef if $AttributeName eq 'ServiceAmount';
            return $AttributeName;
        };
        sub alias{ return 'Alias';}
        sub folder{my $self = shift; return $self;}
        sub children{my $self = shift; return [$self];}
        return bless {}, 'StubShippingLabelDefault';
        ## use critic ( ProhibitNestedSubs ProhibitMultiplePackages ProhibitMagicNumbers )
    }
}

sub StubShippingLabelEmpty {
    {
        ## no critic ( ProhibitNestedSubs ProhibitMultiplePackages ProhibitMagicNumbers )
        package StubShippingLabelEmpty;
        sub get{
            my $self = shift;
            my ($AttributeName) = @_;
            return undef if $AttributeName eq 'TotalWeight';
            return undef if $AttributeName eq 'TotalAmount';
            return undef if $AttributeName eq 'ParcelDepth';
            return undef if $AttributeName eq 'ParcelLength';
            return undef if $AttributeName eq 'ParcelWidth';
            return undef if $AttributeName eq 'LineItemContainer';
            # factor for mm
            return 0.001 if $AttributeName eq 'Factor';
            return 'ServiceType' if $AttributeName eq 'ServiceGroup';
            return undef if $AttributeName eq 'ServiceAmount';
            return $AttributeName;
        };
        sub alias{ return 'Alias';}
        sub folder{my $self = shift; return $self;}
        sub children{my $self = shift; return [$self];}
        return bless {}, 'StubShippingLabelEmpty';
        ## use critic ( ProhibitNestedSubs ProhibitMultiplePackages ProhibitMagicNumbers )
    }
}
RunScript( Sub => \&Main );

1;