use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::DHLIntraship::API::Constants qw (
    $DEFAULTSELECTNAME
    $DHLINTRASHIPSERVICEPATH
);
use DE_EPAGES::DHLIntraship::API::ServiceValidation;
use Test::More;

## no critic ( ProhibitNestedSubs ProhibitMultiplePackages ProhibitPrivateSubroutineUse ProtectPrivateVars)
sub Main {
    TestCreateStates();
    TestCombineOldAndNewStates();
#    RegressionTestProgress();
    TestValidateOptionGroup();
    done_testing();
}

#----------------------------------------------------------------------------------------
sub TestCreateStates{
    my $aInputServices = [
                                { 'Name' => 'GroupOne', 'SelectedOption' => 'ServiceOne', 'Options' => ['one','two'], },
                                { 'Name' => 'GroupOne', 'SelectedOption' => 'ServiceThree', 'Options' => ['one','two'], },
                                { 'Name' => 'GroupOne', 'SelectedOption' => 'ServiceFour', 'Options' => ['one','two'], 'Value'=> {'UserAmount' => 123}},
                                { 'Name' => 'GroupXXX', 'SelectedOption' => $DEFAULTSELECTNAME, 'Options' => ['one','two'], },
                                { 'Name' => 'GroupTwo', 'Options' => ['tree','four'], },
                             ];
    my $ServiceValidation = DE_EPAGES::DHLIntraship::API::ServiceValidation->new( $aInputServices );
    $ServiceValidation->createStates( 'CurrentState' );
    my $aExpected = [
    { 'Name'  => 'GroupOne', 'SelectedOption' => 'ServiceOne', },
    { 'Name'  => 'GroupOne', 'SelectedOption' => 'ServiceThree', },
    { 'Name' => 'GroupOne', 'SelectedOption' => 'ServiceFour', 'UserAmount' => 123}
    ];
    is_deeply($ServiceValidation->{'CurrentState'}, $aExpected, 'createStates - correct transformation from Services struckture to state strukture - Array with states');
}

#----------------------------------------------------------------------------------------
sub TestCombineOldAndNewStates{
    my $aNewStates = [{'Name'=>'GroupOne','SelectedOption' => 'ServiceGroupOne'}];
    my $aOldStates = [ {'Name'=>'GroupOne','SelectedOption' => 'ServiceGroupOne'},
                       {'Name'=>'Grouptwo','SelectedOption' => 'OtherServiceGrouptwo', 'UserAmount' => 123},
                       {'Name'=>'Grouptwo','SelectedOption' => 'OtherServiceGrouptwo2'}
                     ];

    my $ServiceValidation = DE_EPAGES::DHLIntraship::API::ServiceValidation->new();
    my $aResult = $ServiceValidation->combineOldAndNewStates( $aNewStates );
    is_deeply($aResult, $aNewStates, 'combineOldAndNewStates - if only new states there its return the same - Array with states');

    my $aExpected = [
                    {'Name'=>'GroupOne','SelectedOption' => 'ServiceGroupOne'},
                    {'Name'=>'Grouptwo','SelectedOption' => 'OtherServiceGrouptwo', 'UserAmount' => 123},
                    {'Name'=>'Grouptwo','SelectedOption' => 'OtherServiceGrouptwo2'}
                 ];
    $aResult = $ServiceValidation->combineOldAndNewStates( $aNewStates , $aOldStates);
    is_deeply($aResult, $aExpected, 'combineOldAndNewStates - if old and new states there its returns the combination - combinedArray with states');
}

#sub RegressionTestProgress{
#    {
##
##        my $aInputServices = [
##                                { 'Name' => 'GroupOne', 'SelectedOption' => 'option1' },
##                                { 'Name' => 'GroupOne', 'SelectedOption' => 'option2', },
##                                { 'Name' => 'GroupOne', 'SelectedOption' => 'option3', 'Value'=> {'UserAmount' => 123}},
##                                { 'Name' => 'GroupXXX', 'SelectedOption' => $DEFAULTSELECTNAME, 'Options' => ['option1','option2'], },
##                                { 'Name' => 'GroupTwo', 'Options' => ['option1','option2'], },
##                             ];
##        my $ServiceValidationMock = DE_EPAGES::DHLIntraship::API::ServiceValidation->new($aInputServices);
##        local *DE_EPAGES::DHLIntraship::API::ServiceValidation::_getOptionsByGroup = sub {return ['option1','option2','option3']};
##        local *DE_EPAGES::DHLIntraship::API::ServiceValidation::_getOption = sub {{'MinOrMax' => 'MAX', 'MinMaxAmount' => 2500}};
##        local *DE_EPAGES::DHLIntraship::API::ServiceValidation::_canCombineOptions = sub { return 1; };
##        $ServiceValidationMock->{'ServiceTypes'} = ['eins','zwei'];
##        $ServiceValidationMock->{'PackageTypes'} = ['Pack1','Pack2'];
##        $ServiceValidationMock->{'OptionGroups'} = ['GroupOne','GroupTwo'];
##        $ServiceValidationMock->progress();
##
##        is_deeply($ServiceValidationMock->{'ServiceType'},
##                { 'Options' => [ $DEFAULTSELECTNAME, 'eins', 'zwei' ], 'Name' => 'ServiceType', 'SelectedOption' => 'eins' },
##                'progress - Check if ServiceType is correct - ServiceTypeKnot');
##
##        is_deeply($ServiceValidationMock->{'PackageType'},
##                { 'Options' => [ $DEFAULTSELECTNAME, 'Pack1', 'Pack2' ], 'Name' => 'PackageType', 'SelectedOption' => 'Pack1' },
##                'progress - Check if PackageType is correct - PackageTypeKnot');
##
##    use Data::Dumper;
##    print Dumper  $ServiceValidationMock->{'ValidatedOptionGroups'};
##
##        is( scalar @{$ServiceValidationMock->{'ValidatedOptionGroups'}},3,'progress - check if array have the right size - 5');
#return 1;
#    }
sub TestValidateOptionGroup{
    my $ServiceValidationMock = DE_EPAGES::DHLIntraship::API::ServiceValidation->new();
    local *DE_EPAGES::DHLIntraship::API::ServiceValidation::_getOptions = sub { return ['option1','option2','option3']};
    local *DE_EPAGES::DHLIntraship::API::ServiceValidation::_getOptionValue = sub {return {'MinOrMax' => 'MAX', 'MinMaxAmount' => 2500}};
    local *DE_EPAGES::DHLIntraship::API::ServiceValidation::_canCombineOptions = sub { return 1; };
    $ServiceValidationMock->{'CurrentStates'} = [
        {'Name'=>'OptionGroupName','SelectedOption' => 'option1'},
        {'Name'=>'OptionGroupName','SelectedOption' => 'option2'},
        ];
   my $aReturn =  $ServiceValidationMock->_validateOptionGroup('OptionGroupName');
   my $aExpected = {
          'Value' => { 'MinMaxAmount' => 2500, 'MinOrMax' => 'MAX' },
          'Options' => [$DEFAULTSELECTNAME,'option1','option2','option3'],
          'Name' => 'OptionGroupName',
          'SelectedOption' => 'option1'
           };
   is_deeply($aReturn->[0], $aExpected, '_validateOptionGroup - get first element ');
   $aExpected = {
          'Value' => { 'MinMaxAmount' => 2500, 'MinOrMax' => 'MAX' },
          'Options' => [$DEFAULTSELECTNAME,'option2','option3'],
          'Name' => 'OptionGroupName',
          'SelectedOption' => 'option2'
           };
   is_deeply($aReturn->[1], $aExpected, '_validateOptionGroup - get second element ');
   $aExpected = {
          'Options' => [$DEFAULTSELECTNAME,'option3'],
          'Name' => 'OptionGroupName',
           };
   is_deeply($aReturn->[2], $aExpected, '_validateOptionGroup - get thirth element ');
   is(scalar @{$aReturn},3,'_validateOptionGroup - ther should only be three elements ');
}

#sub Test

## use critic ( ProhibitNestedSubs ProhibitMultiplePackages ProhibitPrivateSubroutineUse ProtectPrivateVars)
RunScript( Sub => \&Main );