#!perl -w
use Test::More tests => 6;
use strict;
use File::Temp qw( tempdir );

use DE_EPAGES::Core::API::File qw(CreateDirectory WriteFile DeleteDirectory);
use DE_EPAGES::Core::API::IniConfig;
use DE_EPAGES::Core::API::Script qw(RunScript);
use DE_EPAGES::Installer::API::EpagesJ qw(GetEpagesJBuild GetEpagesJVersion);

sub Main {
    my $TmpDir = tempdir();

    my $Content = <<HERE
epagesj-version: 6.15.3
build: 74
timestamp: 2013-03-12_00-38-45
revision: 18f43b20234aa2f6a810380a64c897c1a9119bb2
ci-url: http://epages7.intern.epages.de:8080/job/ePagesJ%20release/74/
HERE
;

    my $JDir = "$TmpDir/j";
    CreateDirectory("$JDir/webapps");
    WriteFile("$JDir/webapps/epagesj.version", \$Content);

    my $Build = GetEpagesJBuild($JDir);
    my $Version = GetEpagesJVersion($JDir);

    my $ExpectedBuild = '74';
    my $ExpectedVersion = '6.15.3';

    ok( $ExpectedBuild eq $Build, "GetEpagesJBuild: $ExpectedBuild eq $Build" );
    ok( $ExpectedVersion eq $Version, "GetEpagesJVersion: $ExpectedVersion eq $Version" );

    # epagesj.conf may contain duplicate sections, but avoid duplicate keys!
    $Content = <<HERE
[Task]
X=1
Y=1
[Task]
Y=2
Z=2
HERE
;

    WriteFile("$JDir/epagesj.conf", \$Content);

    my $JConf = DE_EPAGES::Core::API::IniConfig->new(
        FileName => "$JDir/epagesj.conf", CaseSensitive => 1);
    ok( $JConf->get('Task','X') eq '1', 'epagesj.conf, section Task, key X');
    ok( $JConf->get('Task','Y') eq "1\n2", 'epagesj.conf, section Task, key Y');
    ok( $JConf->get('Task','Z') eq '2', 'epagesj.conf, section Task, key Z');

    DeleteDirectory($TmpDir, {recursive => 1});

    $JDir = $ENV{EPAGESJ} =~ /\S/ ? $ENV{EPAGESJ} : "$ENV{EPAGES}/j";
  SKIP: {
      skip("file not found: $JDir/conf/epagesj.conf", 1) unless -f "$JDir/conf/epagesj.conf";
    }

    $JConf = DE_EPAGES::Core::API::IniConfig->new(
        FileName => "$JDir/conf/epagesj.conf", CaseSensitive => 1);
    my @Found = sort grep { $_ !~ /^(Java|[^A-Z]+)$/ } @{$JConf->sectionNames};

    my @Expected = qw(
        DSPool
        EPagesJServer
        Mongo
        Search
        ServerThreadPool
        SolrClient
        SolrServer
        Solr
        Task
    );
    my $Missing = join(' ', map { $_ ~~ @Found ? () : $_ } @Expected);
    my $Error = $Missing ? " (missing: $Missing)" : '';
    ok( $Error eq '', "epagesj.conf, section completeness$Error");
}

RunScript( Sub => \&Main );

1;
