#!perl -w
use Test::More tests => 14;

use strict;
use File::Temp qw( tempdir );

use DE_EPAGES::Core::API::Error  qw( GetError ExistsError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::File   qw( CreateDirectory WriteFile );

use DE_EPAGES::Installer::API::Patch qw(
    GetPatchDataURL
    GetPatchDataFile
    DownloadPatchData
    UntarPatchData
);

sub UnderScore {
    my ($String) = @_;
    $String =~ s,\.,_,g;
    return $String;
}

RunScript( Debug => 1, Sub => sub {

    my ($Found, $Expected, $Error);
    my $Version = '6.15.3';
    my $PatchDir = $ENV{EPAGES_PATCHES} ? $ENV{EPAGES_PATCHES} : "$ENV{EPAGES_SHARED}/Patches";

    $Expected = "$PatchDir/patch_". UnderScore($Version). "/epages-$Version-patch-data.tar.gz";
    ok(GetPatchDataFile($Version) eq $Expected, "GetPatchDataFile($Version) -> $Expected");

    $Found = undef;
    eval { $Found = GetPatchDataURL($Version); };
    $Error = ExistsError() ? GetError() : undef;
    ok($Found, "GetPatchDataURL($Version) -> $Found");

    eval { GetPatchDataURL('6.0.9'); };
    $Error = ExistsError() ? GetError() : undef;
    ok($Error, 'GetPatchDataURL(6.0.9) -> '. $Error->message);

    my $TempDir = tempdir( CLEANUP => 1 );
    $ENV{EPAGES_PATCH_DATA_REPO} = $TempDir; ##no critic(RequireLocalizedPunctuationVars)
    CreateDirectory("$TempDir/patch/6.0.9");
    WriteFile("$TempDir/patch/6.0.9/epages-6.0.9-patch-data.tar.gz", \1);

    $Found = undef;
    eval { $Found = GetPatchDataURL('6.0.9'); };
    $Error = ExistsError() ? GetError() : undef;
    ok($Found, "GetPatchDataURL('6.0.9') -> $Found");

    $Found = undef;
    eval { $Found = DownloadPatchData($Version); };
    $Error = ExistsError() ? GetError() : undef;
    ok($Found, "DownloadPatchData($Version) -> $Found");

    $Found = undef;
    eval { $Found = UntarPatchData($Version); };
    $Error = ExistsError() ? GetError() : undef;
    ok($Found, "UntarPatchData($Version) -> $Found");
});
